import networkx as nx
import numpy as np
import itertools
from .transformation import find_transformation
from .frame import Frame


def find_approximation(observations, lock=None):

    unique_frame_ids = np.unique([frame_id for frame_id, _, _ in observations])
    frames = {frame_id: Frame.origin() for frame_id in unique_frame_ids}

    G = nx.empty_graph()
    unique_point_ids = np.unique([p for _, p, _ in observations])
    points0, points1 = {}, {}
    for point_id in unique_point_ids:
        frame_ids = np.unique([f for f, p, _ in observations if p == point_id])
        for pair in itertools.combinations(frame_ids, 2):
            G.add_edge(*pair)
            for f, p, point_ in observations:
                if f == pair[0] and p == point_id:
                    points0[pair] = points0.get(pair, []) + [point_]
                if f == pair[1] and p == point_id:
                    points1[pair] = points1.get(pair, []) + [point_]

    for source_frame, target_frame in nx.dfs_tree(G, source=lock).edges:
        if source_frame < target_frame:
            pointsA = points1[(source_frame, target_frame)]
            pointsB = points0[(source_frame, target_frame)]
        else:
            pointsA = points0[(target_frame, source_frame)]
            pointsB = points1[(target_frame, source_frame)]
        M = find_transformation(pointsA, pointsB)
        frames[target_frame] = Frame(M).transform(frames[source_frame].M)

    return frames
