import numpy as np


class Frame:

    def __init__(self, M):

        self.M = M

    @staticmethod
    def origin():

        return Frame.from_components(0, 0, 0)

    @staticmethod
    def from_components(x, y, yaw):

        return Frame(np.array([
            [np.cos(yaw), -np.sin(yaw), x],
            [np.sin(yaw), np.cos(yaw), y],
            [0, 0, 1],
        ]))

    @property
    def x(self):

        return self.M[0, -1] / self.M[-1, -1]

    @property
    def y(self):

        return self.M[1, -1] / self.M[-1, -1]

    @property
    def yaw(self):

        return np.arctan2(self.M[1, 0], self.M[0, 0])

    def transform(self, M):

        return Frame(M.dot(self.M))

    def invert(self):

        return Frame(np.linalg.inv(self.M))

    def __repr__(self):

        return "Frame(x = %6.3f, y = %6.3f, yaw = %6.1f deg)" % (self.x, self.y, np.rad2deg(self.yaw))
