import numpy as np
from .frame import Frame


def find_transformation(source_points, target_points):

    A = np.stack([point.euk for point in source_points], axis=0)
    B = np.stack([point.euk for point in target_points], axis=0)

    center_A = np.mean(A, axis=0)
    center_B = np.mean(B, axis=0)

    H = (A - center_A).T.dot(B - center_B)
    U, _, V = np.linalg.svd(H)
    R = V.dot(U.T)
    R[:, -1] *= np.sign(np.linalg.det(R))
    Yaw = np.arctan2(R[1, 0], R[0, 0])
    t = center_B - center_A.dot(R.T)

    return Frame.from_components(t[0], t[1], Yaw).M
