import numpy as np
import asyncio
from .actor import Actor
from ..world.image import Image
from ..world.world import World
from ..world.point3d import Point3d
from ..world.pose import Pose
from ..world.marker import Marker
from ..world.calibration import Calibration
from ..world.detection import Detection


class DetectorSimulator(Actor):

    interval: float = 0.01

    async def step(self, world: World):

        for image in reversed(world.images):
            if image.detections is None and image.mac in world.cameras:
                camera = world.cameras[image.mac]
                pose = world.robot.simulation.copy(deep=True)
                await self.detect(image, world.robot.marker, pose, camera.calibration_simulation)
                return

    async def detect(self, image: Image, marker: Marker, pose: Pose, calibration: Calibration):

        await asyncio.sleep(0.5)  # TODO: avoid sleeping in test mode

        image.detections = []

        for category_name, (x, y) in marker.points.items():

            image_point = calibration.project_to_image(Point3d(
                x=pose.x + x * np.cos(pose.yaw) - y * np.sin(pose.yaw),
                y=pose.y + x * np.sin(pose.yaw) + y * np.cos(pose.yaw),
                z=marker.height,
            ))

            radius = 5
            image.detections.append(Detection(
                category_name=category_name,
                model_name='simulation',
                confidence=1.0,
                x=image_point.x - radius,
                y=image_point.y - radius,
                width=2*radius,
                height=2*radius,
            ))
