import numpy as np
from .actor import Actor
from ..world.point import Point
from ..world.world import World


class Tracker(Actor):
    interval: float = 0.1

    async def step(self, world: World):
        if world.tracking == True:
            robot = world.robot.prediction.point
            cameras = list(world.cameras.values())
            distances = [robot.distance(Point(
                x=camera.calibration.extrinsics.translation[0],
                y=camera.calibration.extrinsics.translation[1],
            )) for camera in cameras if camera.calibration is not None]
            if any(distances):
                world.download_queue = [cameras[np.argmin(distances)].mac]
        elif isinstance(world.tracking, list):
            world.download_queue = [mac for mac in world.download_queue if mac in world.tracking]
            world.download_queue += [mac for mac in world.tracking if mac not in world.download_queue]
