import logging
import os
from nicegui import ui
import starlette
import subprocess

bin_dir = os.path.expanduser('~/.rosys/z_cam_binaries/')

log = logging.getLogger('rosys.ota')


def get_binary_filename():
    if not os.path.exists(bin_dir):
        return
    files = os.listdir(bin_dir)
    if any(files):
        files.sort(key=lambda fn: os.path.getmtime(bin_dir + fn), reverse=True)
        return files[0]


def get_ota_binary(request, **kwargs):
    filename = get_binary_filename()
    log.info(f'{request.client.host} requests ota binary with filename: {filename}')
    if filename is None:
        return starlette.responses.Response('No z-cam firmware binary was found', 404)

    file_location = bin_dir + filename
    if not os.path.exists(file_location):
        log.info(f'no file {file_location}')
        return starlette.responses.Response('z-cam firmware does not exists', 404)

    log.info(f'sending {file_location}')
    return starlette.responses.FileResponse(path=file_location, filename=filename)


def get_ota_hash(request, **kwargs):
    log.info(f'{request.client.host} requests ota hash')
    filename = get_binary_filename()
    if filename is None:
        return starlette.responses.Response('No z-cam firmware was found', 404)
    return starlette.responses.Response(content=filename.lstrip('z_cam_').rstrip('.bin'), media_type='text/plain')


ui.add_route(starlette.routing.Route('/ota/binary', get_ota_binary))
ui.add_route(starlette.routing.Route('/ota/hash', get_ota_hash))


def start_legacy_ota_server():
    '''run flask legay server on port 5000 to bring new firmware to old z-cams'''
    try:
        global flask
        import pathlib
        from subprocess import Popen, PIPE
        ota_server_location = pathlib.Path(__file__).parent.parent.resolve() / 'ota_server.py'
        if not ota_server_location.is_file():
            log.error('could not find legacy ota_server.py')
            return
        flask = Popen(str(ota_server_location), stdout=PIPE, stderr=PIPE, shell=True)
        if flask.poll() is None:
            log.info('started legacy ota server on port 5000')
        else:
            out, err = flask.communicate()
            log.info(f'legacy ota server startup failed:\n{out}\{err}')

    except:
        log.exception('could not start legacy ota server')


def stop_legacy_ota_server():
    global flask
    if flask is not None:
        flask.terminate()


ui.on_startup(start_legacy_ota_server)
ui.on_startup(stop_legacy_ota_server)
