from pydantic import BaseModel
from typing import Optional
import numpy as np
from .calibration import Calibration


class CameraNetwork(BaseModel):

    ip: str = None
    inactive_time: float = None
    expected_throughput: float = None
    signal: float = None
    signal_average: float = None


class CameraSynchronization(BaseModel):

    time: float
    micros: float
    offset: float


class Camera(BaseModel):

    mac: str
    network: CameraNetwork = CameraNetwork()
    calibration: Optional[Calibration]
    calibration_simulation: Optional[Calibration]
    projection: Optional[list[list[list[Optional[float]]]]]
    synchronization: Optional[CameraSynchronization]
    locked: bool = False

    @property
    def color(self) -> tuple[int]:
        rgb = np.array([int(self.mac[i:i+2], 16) for i in range(9, 16, 3)])
        rgb = 0.5 * (rgb - rgb.mean()) + rgb.mean()  # NOTE: less saturation
        rgb = (0.5 * rgb + 127).astype(int)  # NOTE: more brightness
        return tuple(rgb.tolist())
