# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rosys',
 'rosys.actors',
 'rosys.actors.calibration',
 'rosys.actors.calibration.helmert',
 'rosys.actors.pathplanning',
 'rosys.automations',
 'rosys.automations.navigation',
 'rosys.ui',
 'rosys.world']

package_data = \
{'': ['*'], 'rosys.ui': ['lib/*']}

install_requires = \
['Pillow>=8.3.0,<9.0.0',
 'aiohttp>=3.7.4,<4.0.0',
 'aioserial>=1.3.0,<2.0.0',
 'asyncio>=3.4.3,<4.0.0',
 'bacs>=0.1.0,<0.2.0',
 'networkx>=2.6.2,<3.0.0',
 'nicegui==0.5.9',
 'numpy>=1.20.1,<2.0.0',
 'opencv-contrib-python-headless>=4.5.2,<5.0.0',
 'pycurl>=7.43.0,<8.0.0',
 'pydantic>=1.8.2,<2.0.0',
 'pyserial>=3.5,<4.0',
 'python-socketio[asyncio_client]>=5.3.0,<6.0.0',
 'requests>=2.25.1,<3.0.0',
 'retry>=0.9.2,<0.10.0',
 'scipy>=1.7.1,<2.0.0',
 'simplejpeg>=1.5.0,<2.0.0',
 'simplejson>=3.17.2,<4.0.0',
 'svgpathtools>=1.4.1,<2.0.0']

setup_kwargs = {
    'name': 'rosys',
    'version': '0.1.22',
    'description': 'Modular Robot System With Elegant Automation Capabilities',
    'long_description': "# RoSys - Zauberzeug Robot System\n\nCurrently RoSys is meant to be run on the Zauberzeug Robot Brain.\n\n## Usage\n\n### Installation\n\n```bash\nsudo apt-get install libcurl4-openssl-dev libssl-dev # required for pycurl\npython3 -m pip install rosys\npython3 -m pip install nicegui # web user interface\n```\n\n## Development\n\n### Starting\n\nThere are some specialities needed to start the RoSys in different environments. To simplify the usage we wrapped this in a script `./docker.sh`.\n\n#### Configuration\n\nYou can configure the system by putting these variables into the `/.env` file:\n\n- ESP_SERIAL=/dev/ttyTHS1: path to the esp devices; default is /dev/null to be able to start RoSys anywhere\n- USE_CAMS=true: weather RoSys should use Zauberzeug Smart Cams for tracking\n\n#### Under The Hood\n\n- there is no host network on Mac so the conf is overwritten (on real robots it's necccessary to scan for cameras)\n- the services will start automatically on jetson when rebooting or crashing to simplify work with real robots\n\n### Continuous Build\n\nWe build with Drone. Run locally with\n\n    drone starlark .drone.star && drone exec --trusted --exclude slack  .drone.yml\n\n### Profiling\n\nIn the system container run\n\n    kernprof profiling.py\n\nto generate the profiling data. View them with the interactive tool\n\n    python -m pstats profiling.py.prof\n",
    'author': 'Zauberzeug GmbH',
    'author_email': 'info@zauberzeug.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/zauberzeug/rosys',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)
